import java.util.*;

/**
 * Auswertung von arithmetischen Postfix-Ausdruecken.
 * 
 * @author Owyn
 * @version 2007-07-20
 */
public class Postfix {

    /**
     * Wertet einen arithmetischen Ausdruck aus.
     * 
     * @param ausdruck Arithmetischer Ausdruck in Postfix-Schreibweise
     * @return das Ergebnis der Rechnung
     */
    private static int eval(String ausdruck) throws PostfixException {
        
        // Ausdruck in Folge von Ziffern, Operatoren und Leerzeichen (jeweils als String) trennen
        StringTokenizer t = new StringTokenizer(ausdruck, " +-*/", true);
        
        // Stack fuer die Auswertung des Ausdrucks
        Stack<Integer> s = new Stack<Integer>();

        int stelle = 0; // String-Index fuer die Ermittlung der Fehlerstelle
        int altestelle = 0; // vorletzte Stelle im String (fuer Fehlerart 3)
        
        while (t.hasMoreTokens()) { // So lange es noch Tokens gibt...
            String token = t.nextToken(); // ...lesen wir einen Token ein...
            char tok = token.charAt(0); // ...und speichern das erste Zeichen als char.
            if (tok >= '0' && tok <= '9') // Wenn dieses eine Ziffer ist, handelt es sich beim Token um eine Zahl...
                s.push(new Integer(token)); // ...die wir als neuen Integer auf den Stack legen.
            else if (tok != ' ') { // Wenn das erste Zeichen weder Ziffer noch Leerzeichen ist, handelt es sich um einen Operator.
                if (s.empty()) throw new PostfixException(ausdruck, stelle, 2);
                int right = s.pop().intValue(); // Also holen wir den rechten...
                if (s.empty()) throw new PostfixException(ausdruck, stelle, 1);
                int left = s.pop().intValue(); // ...und dann den linken Operand vom Stack.
                int result = 0;
                switch (tok) { // Je nach Operator speichern wir das Ergebnis der Rechenoperation...
                    case '+': result = left + right; break;
                    case '-': result = left - right; break;
                    case '*': result = left * right; break;
                    case '/':
                        if (right == 0) throw new PostfixException(ausdruck, stelle, 0);
                        result = left / right;
                        break;
                }
                s.push(new Integer(result)); // ...und legen es als neuen Integer wieder auf den Stack.
            }
            altestelle = stelle;
            stelle += token.length(); // String-Index weiterruecken
        } // An dieser Stelle gibt es keine Tokens mehr...
        int result = s.pop().intValue(); // ...also liegt das Ergebnis des Ausdrucks als oberstes Element auf dem Stack.
        if (!s.empty()) // Wenn jetzt noch etwas auf dem Stack liegt...
            throw new PostfixException(ausdruck, altestelle, 3); // ...gab es zu viele Operanden oder zu wenige Operatoren.
        return result; // Ergebnis zurueckgeben
    }
    
    /**
     * Wertet einen arithmetischen Ausdruck aus und gibt das Ergebnis oder eine Fehlermeldung aus.
     * 
     * @param ausdruck Arithmetischer Ausdruck in Postfix-Schreibweise
     */
    public static void postfix(String ausdruck) {
        
        try {
            System.out.println(ausdruck + " = " + eval(ausdruck));
        }
        catch (PostfixException pe) {
            System.err.println(pe.getMessage());
        }
        
    }

}