/**
 * Exception-Klasse zur Behandlung von Fehlern in Postfix-Ausdruecken.
 * 
 * @author Owyn
 * @version 2007-07-20
 */
public class PostfixException extends Exception {

    String ausdruck;
    int stelle;
    int art;

    /**
     * Erzeugt eine neue PostfixException.
     * 
     * @param stelle Die Stelle im Postfix-Ausdruck (als String-Index), an der der Fehler aufgetreten ist.
     * @param art Die Art des Fehlers: 0 = Division durch 0, 1 = ein Operand fehlt, 2 = beide Operanden fehlen, 3 = zu viele Operanden.
     */
    PostfixException(String ausdruck, int stelle, int art) {
        this.ausdruck = ausdruck;
        this.stelle = stelle;
        this.art = art;
    }
    
    /**
     * Gibt die Fehlermeldung zurueck, inklusive "grafischer" Anzeige der Fehlerstelle
     * 
     * @return aussagekraeftige Fehlermeldung
     */
    public String getMessage() {
        StringBuffer fehler = new StringBuffer();
        switch(art) { // Zuerst die Fehlermeldung...
            case 0:
                fehler.append("Division durch 0!\n");
                break;
            case 1:
                fehler.append("Der Operator erwartet zwei Operanden, es ist aber nur einer da!\n");
                break;
            case 2:
                fehler.append("Der Operator erwartet zwei Operanden, es ist aber keiner da!\n");
                break;
            case 3:
                fehler.append("Zu viele Operanden bzw. zu wenige Operatoren!\n");
                break;
        } // "\n" ist ein Zeilenumbruch
        fehler.append(ausdruck); // ...darunter der urspruengliche Ausdruck...
        fehler.append("\n"); // ...und darunter...
        for (int i = 0; i < stelle; i++) fehler.append(" "); // ...an die richtige Stelle...
        fehler.append("^"); // ...eine Markierung.
        return fehler.toString();
    }

}
